
/* 
 * File: "CoxeterExteions_by_EM_InvolBruhatIterator.h"
 *
 * Author: Eric Marberg
 * Version: June 2013
 * --------------------------------------------------------------
 * Header file for the small helper class InvolBruhatIterator. 
 *
 */



#ifndef __InvolBruhatInterator__
#define __InvolBruhatInterator__



#include "commands.h"
#include <vector>
#include <set>

using namespace std;
using namespace commands;



/*
 * Class: InvolBruhatIterator
 * ---------------------------------------------------------
 * The InvolBruhatIterator class constructor takes three arguments:
 * bruhat_ascents, y, and w.
 *
 * The argument bruhat_ascents is a 2-dimensional array of integer vectors.
 * This is equal to the value of 
 *
 *   TwistedInvolutionsPoset.bruhat_ascents_by_ascents[s]
 *
 * for a particular generator s in S.
 *
 * The arguments y and w are indices of twisted involutions (in the
 * ambient TwistedInvolutionsPoset object). 
 *
 * The constructed InvolBruhatIterator object implements iteration over
 * the set of twisted involutions z with y <= z <= w and s in Des_L(z).
 * The method next() successively returns the indies of such twisted involutions z, 
 * in an order which is compatible with the Bruhat partial order on I_*.
 * 
 */
class InvolBruhatIterator {

	// w, y
	std::vector<int>** bruhat_ascents;
	
	std::set<int> current, following;
	std::set<int>::iterator it;
	
	int w_index;
	Generator s;
	
public:
	
	// The default zero argument constructor returns an empty iterator, 
	// for which hasNext() always returns false.
	//
	InvolBruhatIterator() {}
	
	InvolBruhatIterator(std::vector<int>**, int, int);
    ~InvolBruhatIterator() {}
	
	// Returns false if we've already reached the last iteration
	//
	bool hasNext() const { return !current.empty(); }
		
	int next();	
	
};
	


#endif